/*
This file is part of Telegram Desktop,
the official desktop application for the Telegram messaging service.

For license and copyright information please follow this link:
https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
*/
#include <crl/qt/crl_qt_semaphore.h>

#ifdef CRL_USE_QT

#endif // CRL_USE_QT
