/*
This file is part of Telegram Desktop,
the official desktop application for the Telegram messaging service.

For license and copyright information please follow this link:
https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
*/
#pragma once

#include <crl/crl_semaphore.h>
#include <crl/crl_async.h>
#include <crl/crl_queue.h>
#include <crl/crl_on_main.h>
